@@ -68,9 +68,6 @@
 /* Define to 1 if you have the `atanpi' function. */
 /* #undef HAVE_ATANPI */
 
-/* Define if you have BSD networking headers and libraries. */
-/* #undef HAVE_BSD_NETWORKING */
-
 /* Define to 1 if the compiler understands __builtin_expect. (For intl) */
 #define HAVE_BUILTIN_EXPECT 1
 
@@ -153,14 +150,14 @@
 /* Define to 1 if you have the `ctan' function. */
 #define HAVE_CTAN 1
 
+/* Define to 1 if you have the `ctanh' function. */
+#define HAVE_CTANH 1
+
 /* Define to 1 if you have the <curl/curl.h> header file.
    Set on the command line where supported.
 */
 /* #undef HAVE_CURL_CURL_H */
 
-/* Define to 1 if you have the `ctanh' function. */
-#define HAVE_CTANH 1
-
 /* define if the compiler supports basic C++11 syntax */
 #define HAVE_CXX11 1
 
@@ -375,9 +372,6 @@
 /* Define to 1 if the system has the type `int64_t'. */
 #define HAVE_INT64_T 1
 
-/* Define if you have support for ftp/http access. */
-#define HAVE_INTERNET 1
-
 /* Define if you have the 'intmax_t' type in <stdint.h> or <inttypes.h>. (For
    intl) */
 #define HAVE_INTMAX_T 1
@@ -428,8 +422,10 @@
 /* Define if your <locale.h> file defines LC_MESSAGES. */
 /* #undef HAVE_LC_MESSAGES */
 
-/* Define to 1 if you have the `cc_dynamic' library (-lcc_dynamic). */
-/* #undef HAVE_LIBCC_DYNAMIC */
+/* Define if your system has libcurl >= 7.28.0 with support for https.
+   Set on the command line where supported.
+ */
+/* #undef HAVE_LIBCURL */
 
 /* Define if __libc_stack_end is visible. */
 /* #undef HAVE_LIBC_STACK_END */
@@ -672,9 +668,6 @@
 /* Define to 1 if you have the `snprintf' function. */
 #define HAVE_SNPRINTF 1
 
-/* Define if you have support for sockets. */
-#define HAVE_SOCKETS 1
-
 /* Define to 1 if the system has the type `stack_t'. */
 /* #undef HAVE_STACK_T */
 
@@ -949,8 +942,7 @@
 /* Define if you have 32 bit ints. */
 #define INT_32_BITS 1
 
-/* Define to the sub-directory in which libtool stores uninstalled libraries.
-   */
+/* Define to the sub-directory where libtool stores uninstalled libraries. */
 #define LT_OBJDIR ".libs/"
 
 /* Define if mktime sets errno. */
@@ -1070,6 +1062,11 @@
 /* The size of `size_t', as computed by sizeof. */
 #define SIZEOF_SIZE_T @ST@
 
+/* Workaround for win64 pow() precision issue in Mingw-w64 V3 and higher
+   See http://sourceforge.net/p/mingw-w64/bugs/466 for discussion. */
+#if defined(_WIN64) && defined(__MINGW64_VERSION_MAJOR) && __MINGW64_VERSION_MAJOR >= 3
+#define USE_POWL_IN_R_POW 1
+#endif
 
 /* Define as the maximum value of type 'size_t', if the system doesn't define
    it. (For intl) */
@@ -1086,9 +1083,6 @@
 /* Define to 1 if you have the ANSI C header files. */
 #define STDC_HEADERS 1
 
-/* Define if you provide support for the libxml ftp/http functions. */
-#define SUPPORT_LIBXML 1
-
 /* Define if you have C/C++/Fortran OpenMP support for package code. */
 /* #undef SUPPORT_OPENMP */
 
