@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1998--2015 The R Core Team
+ *  Copyright (C) 1998--2016 The R Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -300,9 +300,9 @@ SEXP attribute_hidden do_fileshow(SEXP call, SEXP op, SEXP args, SEXP rho)
     dl = (Rboolean) asLogical(CAR(args)); args = CDR(args);
     pg = CAR(args);
     n = 0;			/* -Wall */
-    if (!isString(fn) || (n = length(fn)) < 1)
+    if (!isString(fn) || (n = LENGTH(fn)) < 1)
 	error(_("invalid filename specification"));
-    if (!isString(hd) || length(hd) != n)
+    if (!isString(hd) || LENGTH(hd) != n)
 	error(_("invalid '%s' argument"), "headers");
     if (!isString(tl))
 	error(_("invalid '%s' argument"), "title");
@@ -319,11 +319,11 @@ SEXP attribute_hidden do_fileshow(SEXP call, SEXP op, SEXP args, SEXP rho)
 	    f[i] = acopy_string(translateChar(el));
 #endif
 	else
-            error(_("invalid filename specification"));
+	    error(_("invalid filename specification"));
 	if (STRING_ELT(hd, i) != NA_STRING)
 	    h[i] = acopy_string(translateChar(STRING_ELT(hd, i)));
 	else
-            error(_("invalid '%s' argument"), "headers");
+	    error(_("invalid '%s' argument"), "headers");
     }
     if (isValidStringF(tl))
 	t = acopy_string(translateChar(STRING_ELT(tl, 0)));
@@ -331,10 +331,10 @@ SEXP attribute_hidden do_fileshow(SEXP call, SEXP op, SEXP args, SEXP rho)
 	t = "";
     if (isValidStringF(pg)) {
 	SEXP pg0 = STRING_ELT(pg, 0);
-        if (pg0 != NA_STRING)
-            pager = acopy_string(CHAR(pg0));
-        else
-            error(_("invalid '%s' argument"), "pager");
+	if (pg0 != NA_STRING)
+	    pager = acopy_string(CHAR(pg0));
+	else
+	    error(_("invalid '%s' argument"), "pager");
     } else
 	pager = "";
     R_ShowFiles(n, f, h, t, dl, pager);
@@ -384,12 +384,13 @@ SEXP attribute_hidden do_fileappend(SEXP call, SEXP op, SEXP args, SEXP rho)
     int n, n1, n2;
 
     checkArity(op, args);
-    f1 = CAR(args); n1 = length(f1);
-    f2 = CADR(args); n2 = length(f2);
+    f1 = CAR(args);
+    f2 = CADR(args);
     if (!isString(f1))
 	error(_("invalid '%s' argument"), "file1");
     if (!isString(f2))
 	error(_("invalid '%s' argument"), "file2");
+    n1 = LENGTH(f1); n2 = LENGTH(f2);
     if (n1 < 1)
 	error(_("nothing to append to"));
     if (n2 < 1)
@@ -448,7 +449,7 @@ SEXP attribute_hidden do_filecreate(SEXP call, SEXP op, SEXP args, SEXP rho)
 	error(_("invalid filename argument"));
     show = asLogical(CADR(args));
     if (show == NA_LOGICAL) show = 0;
-    n = length(fn);
+    n = LENGTH(fn);
     PROTECT(ans = allocVector(LGLSXP, n));
     for (i = 0; i < n; i++) {
 	LOGICAL(ans)[i] = 0;
@@ -473,7 +474,7 @@ SEXP attribute_hidden do_fileremove(SEXP call, SEXP op, SEXP args, SEXP rho)
     f = CAR(args);
     if (!isString(f))
 	error(_("invalid first filename"));
-    n = length(f);
+    n = LENGTH(f);
     PROTECT(ans = allocVector(LGLSXP, n));
     for (i = 0; i < n; i++) {
 	if (STRING_ELT(f, i) != NA_STRING) {
@@ -539,12 +540,13 @@ SEXP attribute_hidden do_filesymlink(SEXP call, SEXP op, SEXP args, SEXP rho)
     int i;
 #endif
     checkArity(op, args);
-    f1 = CAR(args); n1 = length(f1);
-    f2 = CADR(args); n2 = length(f2);
+    f1 = CAR(args);
+    f2 = CADR(args);
     if (!isString(f1))
 	error(_("invalid first filename"));
     if (!isString(f2))
 	error(_("invalid second filename"));
+    n1 = LENGTH(f1); n2 = LENGTH(f2);
     if (n1 < 1)
 	error(_("nothing to link"));
     if (n2 < 1)
@@ -621,12 +623,13 @@ SEXP attribute_hidden do_filelink(SEXP call, SEXP op, SEXP args, SEXP rho)
     int i;
 #endif
     checkArity(op, args);
-    f1 = CAR(args); n1 = length(f1);
-    f2 = CADR(args); n2 = length(f2);
+    f1 = CAR(args);
+    f2 = CADR(args);
     if (!isString(f1))
 	error(_("invalid first filename"));
     if (!isString(f2))
 	error(_("invalid second filename"));
+    n1 = LENGTH(f1); n2 = LENGTH(f2);
     if (n1 < 1)
 	error(_("nothing to link"));
     if (n2 < 1)
@@ -699,13 +702,14 @@ SEXP attribute_hidden do_filerename(SEXP call, SEXP op, SEXP args, SEXP rho)
 #endif
 
     checkArity(op, args);
-    f1 = CAR(args); n1 = length(f1);
-    f2 = CADR(args); n2 = length(f2);
+    f1 = CAR(args);
+    f2 = CADR(args);
     if (!isString(f1))
 	error(_("invalid '%s' argument"), "from");
     if (!isString(f2))
 	error(_("invalid '%s' argument"), "to");
-    if (n2 != n1)
+    n1 = LENGTH(f1); n2 = LENGTH(f2);
+   if (n2 != n1)
 	error(_("'from' and 'to' are of different lengths"));
     PROTECT(ans = allocVector(LGLSXP, n1));
     for (i = 0; i < n1; i++) {
@@ -794,7 +798,7 @@ SEXP attribute_hidden do_fileinfo(SEXP call, SEXP op, SEXP args, SEXP rho)
     int extras = asInteger(CADR(args));
     if(extras == NA_INTEGER)
 	error(_("invalid '%s' argument"), "extra_cols");
-    int n = length(fn), ncols = 6;
+    int n = LENGTH(fn), ncols = 6;
     if(extras) {
 #ifdef UNIX_EXTRAS
 	ncols = 10;
@@ -842,9 +846,9 @@ SEXP attribute_hidden do_fileinfo(SEXP call, SEXP op, SEXP args, SEXP rho)
 	size_t len = wcslen(wfn);
 	if (len) {
 	    wchar_t *p = wfn + (len - 1);
-            if (len > 1 && (*p == L'/' || *p == L'\\') &&
-	*(p-1) != L':') *p = 0;
-        }
+	    if (len > 1 && (*p == L'/' || *p == L'\\') &&
+		*(p-1) != L':') *p = 0;
+	}
 #else
 	const char *efn = R_ExpandFileName(translateChar(STRING_ELT(fn, i)));
 #endif
@@ -973,7 +977,7 @@ SEXP attribute_hidden do_direxists(SEXP call, SEXP op, SEXP args, SEXP rho)
     fn = CAR(args);
     if (!isString(fn))
 	error(_("invalid filename argument"));
-    int n = length(fn);
+    int n = LENGTH(fn);
     PROTECT(ans = allocVector(LGLSXP, n));
     for (int i = 0; i < n; i++) {
 #ifdef Win32
@@ -985,9 +989,9 @@ SEXP attribute_hidden do_direxists(SEXP call, SEXP op, SEXP args, SEXP rho)
 	size_t len = wcslen(wfn);
 	if (len) {
 	    wchar_t *p = wfn + (len - 1);
-            if (len > 1 && (*p == L'/' || *p == L'\\') &&
-	*(p-1) != L':') *p = 0;
-        }
+	    if (len > 1 && (*p == L'/' || *p == L'\\') &&
+		*(p-1) != L':') *p = 0;
+	}
 #else
 	const char *efn = R_ExpandFileName(translateChar(STRING_ELT(fn, i)));
 #endif
@@ -1119,7 +1123,7 @@ list_files(const char *dnp, const char *stem, int *count, SEXP *pans,
 		    IF_MATCH_ADD_TO_ANS
 	    }
 
-        } // end while()
+	} // end while()
 	closedir(dir);
     }
 }
@@ -1134,9 +1138,9 @@ SEXP attribute_hidden do_listfiles(SEXP call, SEXP op, SEXP args, SEXP rho)
     if (!isString(d)) error(_("invalid '%s' argument"), "path");
     SEXP p = CAR(args); args = CDR(args);
     Rboolean pattern = FALSE;
-    if (isString(p) && length(p) >= 1 && STRING_ELT(p, 0) != NA_STRING)
+    if (isString(p) && LENGTH(p) >= 1 && STRING_ELT(p, 0) != NA_STRING)
 	pattern = TRUE;
-    else if (!isNull(p) && !(isString(p) && length(p) < 1))
+    else if (!isNull(p) && !(isString(p) && LENGTH(p) < 1))
 	error(_("invalid '%s' argument"), "pattern");
     int allfiles = asLogical(CAR(args)); args = CDR(args);
     if (allfiles == NA_LOGICAL)
@@ -1297,7 +1301,7 @@ SEXP attribute_hidden do_fileexists(SEXP call, SEXP op, SEXP args, SEXP rho)
     checkArity(op, args);
     if (!isString(file = CAR(args)))
 	error(_("invalid '%s' argument"), "file");
-    nfile = length(file);
+    nfile = LENGTH(file);
     ans = PROTECT(allocVector(LGLSXP, nfile));
     for (i = 0; i < nfile; i++) {
 	LOGICAL(ans)[i] = 0;
@@ -1355,7 +1359,7 @@ SEXP attribute_hidden do_fileaccess(SEXP call, SEXP op, SEXP args, SEXP rho)
     fn = CAR(args);
     if (!isString(fn))
 	error(_("invalid '%s' argument"), "names");
-    n = length(fn);
+    n = LENGTH(fn);
     mode = asInteger(CADR(args));
     if (mode < 0 || mode > 7) error(_("invalid '%s' argument"), "mode");
     modemask = 0;
@@ -1633,7 +1637,7 @@ SEXP attribute_hidden do_unlink(SEXP call, SEXP op, SEXP args, SEXP env)
 		globfree(&globbuf);
 	    } else failures++;
 #else /* HAVE_GLOB */
-	        failures += R_unlink(names, recursive, force);
+		failures += R_unlink(names, recursive, force);
 	    } else failures++;
 #endif
 	}
@@ -1642,69 +1646,6 @@ SEXP attribute_hidden do_unlink(SEXP call, SEXP op, SEXP args, SEXP env)
 }
 #endif
 
-#if 0
-static void chmod_one(const char *name)
-{
-    DIR *dir;
-    struct dirent *de;
-    char p[PATH_MAX];
-#ifdef Win32
-    struct _stati64 sb;
-#else
-    struct stat sb;
-#endif
-#ifndef Win32
-    mode_t mask = S_IRUSR | S_IRGRP | S_IROTH | S_IWUSR, /* 0644 */
-	dirmask = mask | S_IXUSR | S_IXGRP | S_IXOTH; /* 0755 */
-#endif
-
-    if (streql(name, ".") || streql(name, "..")) return;
-    if (!R_FileExists(name)) return;
-#ifdef Win32
-    _stati64(name, &sb);
-    chmod(name, _S_IWRITE);
-#else
-    stat(name, &sb);
-    chmod(name, (sb.st_mode | mask) & dirmask);
-#endif
-    if ((sb.st_mode & S_IFDIR) > 0) { /* a directory */
-#ifndef Win32
-	chmod(name, dirmask);
-#endif
-	if ((dir = opendir(name)) != NULL) {
-	    while ((de = readdir(dir))) {
-		if (streql(de->d_name, ".") || streql(de->d_name, ".."))
-		    continue;
-		size_t n = strlen(name);
-		if (name[n-1] == R_FileSep[0])
-		    snprintf(p, PATH_MAX, "%s%s", name, de->d_name);
-		else
-		    snprintf(p, PATH_MAX, "%s%s%s", name, R_FileSep, de->d_name);
-		chmod_one(p);
-	    }
-	    closedir(dir);
-	} else {
-	    /* we were unable to read a dir */
-	}
-    }
-}
-
-/* recursively fix up permissions: used for R CMD INSTALL and build.
-   NB: this overrides umask. */
-SEXP attribute_hidden do_dirchmod(SEXP call, SEXP op, SEXP args, SEXP env)
-{
-    SEXP dr;
-    checkArity(op, args);
-    dr = CAR(args);
-    if(!isString(dr) || length(dr) != 1)
-	error(_("invalid '%s' argument"), "dir");
-    chmod_one(translateChar(STRING_ELT(dr, 0)));
-
-    return R_NilValue;
-}
-#endif
-
-
 SEXP attribute_hidden do_getlocale(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     int cat;
@@ -1843,6 +1784,7 @@ SEXP attribute_hidden do_localeconv(SEXP call, SEXP op, SEXP args, SEXP rho)
     int i = 0;
     char buff[20];
 
+    checkArity(op, args);
     PROTECT(ans = allocVector(STRSXP, 18));
     PROTECT(ansnames = allocVector(STRSXP, 18));
     SET_STRING_ELT(ans, i, mkChar(lc->decimal_point));
@@ -1904,13 +1846,13 @@ SEXP attribute_hidden do_pathexpand(SEXP call, SEXP op, SEXP args, SEXP rho)
     fn = CAR(args);
     if (!isString(fn))
 	error(_("invalid '%s' argument"), "path");
-    n = length(fn);
+    n = LENGTH(fn);
     PROTECT(ans = allocVector(STRSXP, n));
     for (i = 0; i < n; i++) {
-        SEXP tmp = STRING_ELT(fn, i);
-        if (tmp != NA_STRING) {
-            tmp = markKnown(R_ExpandFileName(translateChar(tmp)), tmp);
-        }
+	SEXP tmp = STRING_ELT(fn, i);
+	if (tmp != NA_STRING) {
+	    tmp = markKnown(R_ExpandFileName(translateChar(tmp)), tmp);
+	}
 	SET_STRING_ELT(ans, i, tmp);
     }
     UNPROTECT(1);
@@ -1944,6 +1886,7 @@ static Rboolean R_can_use_X11(void)
 /* only actually used on Unix */
 SEXP attribute_hidden do_capabilitiesX11(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
+    checkArity(op, args);
 #ifdef Unix
     return ScalarLogical(R_can_use_X11());
 #else
@@ -2027,25 +1970,13 @@ SEXP attribute_hidden do_capabilities(SEXP call, SEXP op, SEXP args, SEXP rho)
 #endif
 
     SET_STRING_ELT(ansnames, i, mkChar("http/ftp"));
-#if HAVE_INTERNET
     LOGICAL(ans)[i++] = TRUE;
-#else
-    LOGICAL(ans)[i++] = FALSE;
-#endif
 
     SET_STRING_ELT(ansnames, i, mkChar("sockets"));
-#ifdef HAVE_SOCKETS
     LOGICAL(ans)[i++] = TRUE;
-#else
-    LOGICAL(ans)[i++] = FALSE;
-#endif
 
     SET_STRING_ELT(ansnames, i, mkChar("libxml"));
-#ifdef SUPPORT_LIBXML
     LOGICAL(ans)[i++] = TRUE;
-#else
-    LOGICAL(ans)[i++] = FALSE;
-#endif
 
     SET_STRING_ELT(ansnames, i, mkChar("fifo"));
 #if (defined(HAVE_MKFIFO) && defined(HAVE_FCNTL_H)) || defined(_WIN32)
@@ -2119,7 +2050,7 @@ SEXP attribute_hidden do_capabilities(SEXP call, SEXP op, SEXP args, SEXP rho)
     LOGICAL(ans)[i++] = sizeof(LDOUBLE) > sizeof(double);
 
     SET_STRING_ELT(ansnames, i, mkChar("libcurl"));
-#ifdef HAVE_CURL_CURL_H
+#ifdef HAVE_LIBCURL
     LOGICAL(ans)[i++] = TRUE;
 #else
     LOGICAL(ans)[i++] = FALSE;
@@ -2131,47 +2062,6 @@ SEXP attribute_hidden do_capabilities(SEXP call, SEXP op, SEXP args, SEXP rho)
     return ans;
 }
 
-#if defined(HAVE_BSD_NETWORKING) && defined(HAVE_ARPA_INET_H)
-#include <netdb.h>
-#include <sys/socket.h>
-#include <netinet/in.h>
-#include <arpa/inet.h>
-
-SEXP attribute_hidden do_nsl(SEXP call, SEXP op, SEXP args, SEXP rho)
-{
-    SEXP ans = R_NilValue;
-    const char *name; char ip[] = "xxx.xxx.xxx.xxx";
-    struct hostent *hp;
-
-    checkArity(op, args);
-    if (!isString(CAR(args)) || length(CAR(args)) != 1)
-	error(_("'hostname' must be a character vector of length 1"));
-    name = translateChar(STRING_ELT(CAR(args), 0));
-
-    hp = gethostbyname(name);
-
-    if (hp == NULL) {		/* cannot resolve the address */
-	warning(_("nsl() was unable to resolve host '%s'"), name);
-    } else {
-	if (hp->h_addrtype == AF_INET) {
-	    struct in_addr in;
-	    memcpy(&in.s_addr, *(hp->h_addr_list), sizeof (in.s_addr));
-	    strcpy(ip, inet_ntoa(in));
-	} else {
-	    warning(_("unknown format returned by C function 'gethostbyname'"));
-	}
-	ans = mkString(ip);
-    }
-    return ans;
-}
-#else
-SEXP attribute_hidden do_nsl(SEXP call, SEXP op, SEXP args, SEXP rho)
-{
-    warning(_("nsl() is not supported on this platform"));
-    return R_NilValue;
-}
-#endif
-
 SEXP attribute_hidden do_sysgetpid(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     checkArity(op, args);
@@ -2195,7 +2085,7 @@ SEXP attribute_hidden do_dircreate(SEXP call, SEXP op, SEXP args, SEXP env)
 
     checkArity(op, args);
     path = CAR(args);
-    if (!isString(path) || length(path) != 1)
+    if (!isString(path) || LENGTH(path) != 1)
 	error(_("invalid '%s' argument"), "path");
     if (STRING_ELT(path, 0) == NA_STRING) return ScalarLogical(FALSE);
     show = asLogical(CADR(args));
@@ -2255,7 +2145,7 @@ SEXP attribute_hidden do_dircreate(SEXP call, SEXP op, SEXP args, SEXP env)
 
     checkArity(op, args);
     path = CAR(args);
-    if (!isString(path) || length(path) != 1)
+    if (!isString(path) || LENGTH(path) != 1)
 	error(_("invalid '%s' argument"), "path");
     if (STRING_ELT(path, 0) == NA_STRING) return ScalarLogical(FALSE);
     show = asLogical(CADR(args));
@@ -2411,6 +2301,9 @@ static int do_copy(const wchar_t* from, const wchar_t* name, const wchar_t* to,
 		nfail++;
 		goto copy_error;
 	    }
+	} else if (!over) {
+	  nfail++;
+	  goto copy_error;
 	}
 	if(fp1) fclose(fp1); fp1 = NULL;
 	if(fp2) fclose(fp2); fp2 = NULL;
@@ -2444,7 +2337,7 @@ SEXP attribute_hidden do_filecopy(SEXP call, SEXP op, SEXP args, SEXP rho)
 	    error(_("invalid '%s' argument"), "to");
 	over = asLogical(CAR(args)); args = CDR(args);
 	if (over == NA_LOGICAL)
-	    error(_("invalid '%s' argument"), "over");
+	    error(_("invalid '%s' argument"), "overwrite");
 	recursive = asLogical(CAR(args)); args = CDR(args);
 	if (recursive == NA_LOGICAL)
 	    error(_("invalid '%s' argument"), "recursive");
@@ -2457,7 +2350,7 @@ SEXP attribute_hidden do_filecopy(SEXP call, SEXP op, SEXP args, SEXP rho)
 	wcsncpy(dir,
 		filenameToWchar(STRING_ELT(to, 0), TRUE),
 		PATH_MAX);
-        dir[PATH_MAX - 1] = L'\0';
+	dir[PATH_MAX - 1] = L'\0';
 	if (*(dir + (wcslen(dir) - 1)) !=  L'\\')
 	    wcsncat(dir, L"\\", PATH_MAX);
 	for (i = 0; i < nfiles; i++) {
@@ -2633,10 +2526,11 @@ static int do_copy(const char* from, const char* name, const char* to,
 		nfail++;
 		goto copy_error;
 	    }
-	    if(fp2) fclose(fp2); fp2 = NULL;
+	    if(fp2) {fclose(fp2); fp2 = NULL;}
 	    if(perms) chmod(dest, sb.st_mode & mask);
 	    if(dates) copyFileTime(this, dest);
-	}
+	} else if (!over)
+	    nfail++;
 copy_error:
 	if(fp2) fclose(fp2);
 	if(fp1) fclose(fp1);
@@ -2664,7 +2558,7 @@ SEXP attribute_hidden do_filecopy(SEXP call, SEXP op, SEXP args, SEXP rho)
 	    error(_("invalid '%s' argument"), "to");
 	over = asLogical(CAR(args)); args = CDR(args);
 	if (over == NA_LOGICAL)
-	    error(_("invalid '%s' argument"), "over");
+	    error(_("invalid '%s' argument"), "overwrite");
 	recursive = asLogical(CAR(args)); args = CDR(args);
 	if (recursive == NA_LOGICAL)
 	    error(_("invalid '%s' argument"), "recursive");
@@ -2677,7 +2571,7 @@ SEXP attribute_hidden do_filecopy(SEXP call, SEXP op, SEXP args, SEXP rho)
 	strncpy(dir,
 		R_ExpandFileName(translateChar(STRING_ELT(to, 0))),
 		PATH_MAX);
-        dir[PATH_MAX - 1] = '\0';
+	dir[PATH_MAX - 1] = '\0';
 	if (*(dir + (strlen(dir) - 1)) !=  '/')
 	    strncat(dir, "/", PATH_MAX);
 	for (i = 0; i < nfiles; i++) {
@@ -2685,7 +2579,7 @@ SEXP attribute_hidden do_filecopy(SEXP call, SEXP op, SEXP args, SEXP rho)
 		strncpy(from,
 			R_ExpandFileName(translateChar(STRING_ELT(fn, i))),
 			PATH_MAX);
-                from[PATH_MAX - 1] = '\0';
+		from[PATH_MAX - 1] = '\0';
 		size_t ll = strlen(from);
 		if (ll) {  // people do pass ""
 		    /* If there is a trailing sep, this is a mistake */
@@ -2694,11 +2588,11 @@ SEXP attribute_hidden do_filecopy(SEXP call, SEXP op, SEXP args, SEXP rho)
 		    p = strrchr(from, '/') ;
 		    if (p) {
 			strncpy(name, p+1, PATH_MAX);
-                        name[PATH_MAX - 1] = '\0';
+			name[PATH_MAX - 1] = '\0';
 			*(p+1) = '\0';
 		    } else {
 			strncpy(name, from, PATH_MAX);
-                        name[PATH_MAX - 1] = '\0';
+			name[PATH_MAX - 1] = '\0';
 			strncpy(from, "./", PATH_MAX);
 		    }
 		    nfail = do_copy(from, name, dir, over, recursive,
@@ -3025,12 +2919,20 @@ void u_getVersion(UVersionInfo versionArray);
 # include <gnu/libc-version.h>
 #endif
 
+#ifdef HAVE_LIBREADLINE
+# ifdef HAVE_READLINE_READLINE_H
+#  include <readline/readline.h>
+# else
+extern const char *rl_library_version;
+# endif
+#endif
+
 SEXP attribute_hidden
 do_eSoftVersion(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     checkArity(op, args);
-    SEXP ans = PROTECT(allocVector(STRSXP, 7));
-    SEXP nms = PROTECT(allocVector(STRSXP, 7));
+    SEXP ans = PROTECT(allocVector(STRSXP, 8));
+    SEXP nms = PROTECT(allocVector(STRSXP, 8));
     setAttrib(ans, R_NamesSymbol, nms);
     unsigned int i = 0;
     char p[256];
@@ -3073,6 +2975,12 @@ do_eSoftVersion(SEXP call, SEXP op, SEXP args, SEXP rho)
 #endif
     SET_STRING_ELT(ans, i, mkChar(p));
     SET_STRING_ELT(nms, i++, mkChar("iconv"));
+#ifdef HAVE_LIBREADLINE
+    SET_STRING_ELT(ans, i, mkChar(rl_library_version));
+#else
+    SET_STRING_ELT(ans, i, mkChar(""));
+#endif
+    SET_STRING_ELT(nms, i++, mkChar("readline"));
     UNPROTECT(2);
     return ans;
 }
